unit Qrepedit;
{
  Quick Report Editor

  Copyright (c) 1997 Demidov Andre (andre@lion.tu-chel.ac.ru)
                V1.1

  It is FREEWARE component and must be FREEWARE after
     your modifications.
  If you modify this component then
     send your modifications to autor, please.

}
interface

{$DEFINE QR20}
uses
  Classes, Controls,
{$IFDEF QR20}
  QuickRpt, QRCtrls, QRPrntr,
{$ELSE}
  QuickRep,
{$ENDIF}
  Menus, winTypes, extCtrls;

type
  string32 = string[32];
  TFormatValueType=(fvInteger,fvFloat,fvDateTime,fvString);
  TAutoResizeType=(arNoChange,arChangeSize,arChangePos);

  TAQRLabel = class(TQRLabel)
    private
      FFieldName:string32;
      FFieldFormat:string32;
      FAutoResize:TAutoResizeType;
      function FieldNameNotNull:boolean;
      function FieldFormatNotNull:boolean;
    protected
      property OnMouseDown;
      property OnMouseUp;
      property OnMouseMove;
{$IFDEF QR20}
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
      procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
{$ENDIF}
    public
      constructor Create(AOwner: TComponent); override;
    published
      property FieldName:string32 read FFieldName write FFieldName stored FieldNameNotNull;
      property FieldFormat:string32 read FFieldFormat write FFieldFormat stored FieldFormatNotNull;
      property AutoResize:TAutoResizeType read FAutoResize write FAutoResize default arNoChange;
  end;

  TAQRShape = class(TQRShape)
    private
      FAutoResize:TAutoResizeType;
    protected
      property OnMouseDown;
      property OnMouseUp;
      property OnMouseMove;
{$IFDEF QR20}
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
{$ENDIF}
    public
      constructor Create(AOwner: TComponent); override;
    published
      property AutoResize:TAutoResizeType read FAutoResize write FAutoResize default arNoChange;
  end;

  TAQRMemo = class(TQRMemo)
    private
      FFieldName:string32;
      FFieldFormat:string32;
      FAutoResize:TAutoResizeType;
      function FieldNameNotNull:boolean;
      function FieldFormatNotNull:boolean;
    protected
      property OnMouseDown;
      property OnMouseUp;
      property OnMouseMove;
{$IFDEF QR20}
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
      procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
{$ENDIF}
    public
      constructor Create(AOwner: TComponent); override;
      function GetTextHeight:integer;
    published
      property FieldName:string32 read FFieldName write FFieldName stored FieldNameNotNull;
      property FieldFormat:string32 read FFieldFormat write FFieldFormat stored FieldFormatNotNull;
      property AutoResize:TAutoResizeType read FAutoResize write FAutoResize default arNoChange;
  end;

  TAQRBand = class(TQRBand)
    private
      function GetHeightAutoResize:integer;
      procedure SetHeightAutoResize(NewHeight:integer);
    protected
      property OnMouseDown;
      property OnMouseUp;
      property OnMouseMove;
{$IFDEF QR20}
    public
      procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
      procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
      procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer); override;
{$ENDIF}
    published
      property HeightAutoResize:integer read GetHeightAutoResize write SetHeightAutoResize default 0;
  end;

  TFormatValueEvent = procedure(Sender:TObject; const FormatStr:string;
       ValueType:TFormatValueType; ValuePtr:Pointer; var ResultStr:string) of object;
  TGetFieldNamesEvent = procedure(Sender:TObject; FieldNames:TStrings) of object;
  TGetFieldFormatsEvent = procedure(Sender:TObject; const FieldName:string;
       FieldFormats:TStrings) of object;

  TAQREditor = class(TComponent)
  private
    MainMenu:TMainMenu;
    PopupMenu:TPopupMenu;
    MoveControl:TControl;
    FrameControl:TShape;
    PopupControl:TControl;
    MoveX,moveY:integer;
    MoveRect:Trect;
    MoveMode:integer;
    EditMode:integer;
    CreateClass:integer;
    FOnFormatValue:TFormatValueEvent;
    FOnGetFieldNames:TGetFieldNamesEvent;
    FOnGetFieldFormats:TGetFieldFormatsEvent;
    FOnPrintPreview:TNotifyEvent;
    FOnCreateMenu:TNotifyEvent;
    FFileName:string;
    procedure EditControl(Sender: TControl);
    procedure EnableEdit;
    procedure DisableEdit;
    procedure GetFormats(Sender:TObject);
    procedure ControlMouseDown(Sender: TObject; Button: TMouseButton;
       Shift: TShiftState; X, Y: Integer);
    procedure ControlMouseMove(Sender: TObject; Shift: TShiftState;
        X, Y: Integer);
    procedure ControlMouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
    procedure PopupMoveClick(Sender:TObject);
    procedure PopupChangeClick(Sender:TObject);
    procedure PopupDeleteClick(Sender:TObject);
    procedure PopupToBackClick(Sender:TObject);
    procedure PopupToFrontClick(Sender:TObject);
    procedure FileOpenClick(Sender:TObject);
    procedure FileSaveClick(Sender:TObject);
    procedure FilePrinterClick(Sender:TObject);
    procedure FilePrintClick(Sender:TObject);
    procedure FilePageClick(Sender:TObject);
    procedure FileCloseClick(Sender:TObject);
    procedure EditCreateLabelClick(Sender:TObject);
    procedure EditCreateShapeClick(Sender:TObject);
    procedure EditCreateMemoClick(Sender:TObject);
    procedure EditPropertyClick(Sender:TObject);
    procedure EditMoveClick(Sender:TObject);
    procedure EditDeleteClick(Sender:TObject);
  protected
    procedure DoFormatValue(const FormatStr:string;
       ValueType:TFormatValueType; ValuePtr:Pointer; var ResultStr:string);
         virtual;
  public
    procedure DefaultFormatValue(const FormatStr:string;
       ValueType:TFormatValueType; ValuePtr:Pointer; var ResultStr:String);
         virtual;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure EditReport;
    procedure LoadReport;
    procedure SetFieldValue(const FieldName:string; ValueType:TFormatValueType; ValuePtr:Pointer);
    procedure SetFieldInteger(const FieldName:string; Value:longint);
    procedure SetFieldFloat(const FieldName:string; Value:extended);
    procedure SetFieldString(const FieldName:string; const Value:string);
    procedure SetFieldDateTime(const FieldName:string; Value:TDateTime);
  published
    property FileName:string read FFileName write FFileName;
    property OnFormatValue:TFormatValueEvent read FOnFormatValue write FOnFormatValue;
    property OnGetFieldNames:TGetFieldNamesEvent read FOnGetFieldNames write FOnGetFieldNames;
    property OnGetFieldFormats:TGetFieldFormatsEvent read FOnGetFieldFormats write FOnGetFieldFormats;
    property OnPrintPreview:TNotifyEvent read FOnPrintPreview write FOnPrintPreview;
    property OnCreateMenu:TNotifyEvent read FOnCreateMenu write FOnCreateMenu;
  end;

procedure Register;

implementation
uses editpage,editband,editlabl,editshap,editmemo, sysutils, dialogs,
   Forms, stdCtrls, Graphics, winprocs, qedconst;

{$IFDEF WIN32}
{$R *.D32}
{$ELSE}
{$R *.D16}
{$ENDIF}

procedure Register;
begin
{$IFDEF WIN32}
  RegisterComponents('QReport', [TAQREditor,TAQRBand,TAQRLabel,TAQRShape,TAQRMemo]);
{$ELSE}
  RegisterComponents('QuickReport', [TAQREditor,TAQRBand,TAQRLabel,TAQRShape,TAQRMemo]);
{$ENDIF}
end;

{ TAQRLabel }

constructor TAQRLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFieldName:='';
  FFieldFormat:='';
  FAutoResize:=arNoChange;
end;

{$IFDEF QR20}
procedure TAQRLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseDown) then
    OnMouseDown(Self,Button,Shift,X,Y)
  else
    inherited MouseDown(Button,Shift,X,Y);
end;
procedure TAQRLabel.MouseMove(Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseMove) then
    OnMouseMove(Self,Shift,X,Y)
  else
    inherited MouseMove(Shift,X,Y);
end;
procedure TAQRLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseUp) then
    OnMouseUp(Self,Button,Shift,X,Y)
  else
    inherited MouseUp(Button,Shift,X,Y);
end;
{$ENDIF}

function TAQRLabel.FieldNameNotNull:boolean;
begin
  Result:=Length(FFieldName)<>0;
end;

function TAQRLabel.FieldFormatNotNull:boolean;
begin
  Result:=Length(FFieldFormat)<>0;
end;

{ TAQRMemo }

constructor TAQRMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFieldName:='';
  FFieldFormat:='';
  FAutoResize:=arNoChange;
  AutoSize:=False;
end;

{$IFDEF QR20}
procedure TAQRMemo.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseDown) then
    OnMouseDown(Self,Button,Shift,X,Y)
  else
    inherited MouseDown(Button,Shift,X,Y);
end;
procedure TAQRMemo.MouseMove(Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseMove) then
    OnMouseMove(Self,Shift,X,Y)
  else
    inherited MouseMove(Shift,X,Y);
end;
procedure TAQRMemo.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseUp) then
    OnMouseUp(Self,Button,Shift,X,Y)
  else
    inherited MouseUp(Button,Shift,X,Y);
end;
{$ENDIF}

function TAQRMemo.FieldNameNotNull:boolean;
begin
  Result:=Length(FFieldName)<>0;
end;

function TAQRMemo.FieldFormatNotNull:boolean;
begin
  Result:=Length(FFieldFormat)<>0;
end;

function TAQRMemo.GetTextHeight:integer;
var oldFont:TFont;
    TextBounds:TRect;
    BufLen: Word;
    I: Integer;
    StrEnd, StrTxt: PChar;
    StrBuf: array[0..255] of Char;
begin
  BufLen := 1;
  for I := 0 to Lines.Count - 1 do
    Inc(BufLen, Length(Lines.Strings[I]) + 1);
  StrTxt := StrAlloc(BufLen);
  try
    StrEnd := StrTxt;
    for I := 0 to Lines.Count - 1 do
    begin
      StrPCopy(StrBuf, Lines.Strings[I]);
      StrEnd := StrECopy(StrEnd, StrBuf);
      StrEnd := StrECopy(StrEnd, #10);
    end;
    with (Owner as TForm) do
    begin
      OldFont:=Canvas.Font;
      Canvas.Font:=Self.Font;
      TextBounds := Rect(0, 0, Self.ClientWidth, 0);
      DrawText(Canvas.Handle, StrTxt, -1, TextBounds, DT_CALCRECT);
      Canvas.Font:=oldFont;
    end;
    Result:=TextBounds.Bottom;
  finally
    StrDispose(StrTxt);
  end;
end;

{ TAQRShape }

constructor TAQRShape.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FAutoResize:=arNoChange;
end;

{$IFDEF QR20}
procedure TAQRShape.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseDown) then
    OnMouseDown(Self,Button,Shift,X,Y)
  else
    inherited MouseDown(Button,Shift,X,Y);
end;
procedure TAQRShape.MouseMove(Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseMove) then
    OnMouseMove(Self,Shift,X,Y)
  else
    inherited MouseMove(Shift,X,Y);
end;
procedure TAQRShape.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseUp) then
    OnMouseUp(Self,Button,Shift,X,Y)
  else
    inherited MouseUp(Button,Shift,X,Y);
end;
{$ENDIF}

{ TAQRBand }

{$IFDEF QR20}
procedure TAQRBand.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseDown) then
    OnMouseDown(Self,Button,Shift,X,Y)
  else
    inherited MouseDown(Button,Shift,X,Y);
end;
procedure TAQRBand.MouseMove(Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseMove) then
    OnMouseMove(Self,Shift,X,Y)
  else
    inherited MouseMove(Shift,X,Y);
end;
procedure TAQRBand.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin
  if Assigned(OnMouseUp) then
    OnMouseUp(Self,Button,Shift,X,Y)
  else
    inherited MouseUp(Button,Shift,X,Y);
end;
{$ENDIF}

procedure TAQRBand.SetHeightAutoResize(NewHeight:integer);
var d,i:integer;
begin
  d:=NewHeight-Height;
  if d=0 then Exit;
  Height:=NewHeight;
  for i:=0 to ControlCount-1 do
  begin
    if Controls[i] is TAQRLabel then
    begin
      with Controls[i] as TAQRLabel do
      begin
        case AutoResize of
          arChangePos: Top:=Top+d;
          arChangeSize: Height:=Height+d;
        end;
      end;
    end
    else if Controls[i] is TAQRMemo then
    begin
      with Controls[i] as TAQRMemo do
      begin
        case AutoResize of
          arChangePos: Top:=Top+d;
          arChangeSize: Height:=Height+d;
        end;
      end;
    end
    else if Controls[i] is TAQRShape then
    begin
      with Controls[i] as TAQRShape do
      begin
        case AutoResize of
          arChangePos: Top:=Top+d;
          arChangeSize: Height:=Height+d;
        end;
      end;
    end;
  end;
end;

function TAQRBand.GetHeightAutoResize:integer;
begin
  Result:=Height;
end;

{ TAQREditor }

constructor TAQREditor.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  MainMenu:=nil;
  PopupMenu:=nil;
  MoveControl:=nil;
  FrameControl:=TShape.Create(Self);
  with FrameControl do
  begin
    Brush.Style:=bsClear;
    Pen.Color:=clRed;
    Parent:=nil;
  end;
end;

destructor TAQREditor.Destroy;
begin
  if owner<>nil then
  begin
  disableEdit;
  with Owner as TForm do
    if Menu=MainMenu then
      Menu:=nil;
  end;
  inherited Destroy;
end;

procedure TAQREditor.GetFormats(Sender: TObject);
Var lblfmt,lblfld: TComboBox;
begin
  lblfld:=(Sender as TComboBox);
  lblfmt:=(lblfld.Owner.FindComponent('LabelFormat')) as TComboBox;
  lblfmt.items.Clear;
  if Assigned(FOnGetFieldFormats) then
    FOnGetFieldFormats(Self,lblfld.text,lblfmt.items);
end;

procedure TAQREditor.EditControl(Sender: TControl);
var lbl:TAQRLabel;
    shp:TAQRShape;
    mem:TAQRMemo;
begin
  if Sender is TAQRLabel then
  begin
    lbl:=(Sender as TAQRLabel);
    if QRLabelSetupFrm=nil then
      Application.CreateForm(TQRLabelSetupFrm, QRLabelSetupFrm);
    with QRLabelSetupFrm do
    begin
      labeltext.text:=lbl.Caption;
      FontSetup.Font:=lbl.Font;
      labelleft.text:=inttostr(lbl.left);
      labeltop.text:=inttostr(lbl.top);
      labelwidth.text:=inttostr(lbl.width);
      labelheight.text:=inttostr(lbl.height);
      labelfield.text:=lbl.fieldname;
      labelformat.text:=lbl.fieldformat;
      labeltransporent.checked:=lbl.transparent;
      labelautosize.checked:=lbl.autosize;
      labelaligment.itemindex:=ord(lbl.alignment);
      labelfield.items.Clear;
      if Assigned(FOnGetFieldNames) then
        FOnGetFieldNames(Self,labelfield.items);
      labelfield.onExit:=Self.GetFormats;
      GetFormats(labelField);
      ShowModal;
      labelfield.onExit:=nil;
      if ModalResult=mrOk then
      begin
        lbl.Font:=FontSetup.Font;
        lbl.left:=strtoint(labelleft.text);
        lbl.top:=strtoint(labeltop.text);
        lbl.width:=strtoint(labelwidth.text);
        lbl.height:=strtoint(labelheight.text);
        lbl.autosize:=labelautosize.checked;
        lbl.fieldname:=labelfield.text;
        lbl.fieldformat:=labelformat.text;
        lbl.transparent:=labeltransporent.checked;
        lbl.alignment:=Talignment(labelaligment.itemindex);
        lbl.caption:=labeltext.text;
      end;
    end;
  end
  else if Sender is TAQRShape then
  begin
    shp:=(Sender as TAQRShape);
    if QRShapeSetupFrm=nil then
      Application.CreateForm(TQRShapeSetupFrm, QRShapeSetupFrm);
    with QRShapeSetupFrm do
    begin
      shapeleft.text:=inttostr(shp.left);
      shapetop.text:=inttostr(shp.top);
      shapewidth.text:=inttostr(shp.width);
      shapeheight.text:=inttostr(shp.height);
      shapeStyle.itemindex:=ord(shp.shape);
      PenColor.ColorValue:=shp.pen.color;
      BrushColor.ColorValue:=shp.Brush.color;
      PenWidth.text:=inttostr(shp.pen.width);
      PenStyle:=shp.pen.style;
      BrushStyle:=shp.brush.style;
      ShowModal;
      if ModalResult=mrOk then
      begin
        shp.left:=strtoint(shapeleft.text);
        shp.top:=strtoint(shapetop.text);
        shp.width:=strtoint(shapewidth.text);
        shp.height:=strtoint(shapeheight.text);
        shp.shape:=TQRShapeType(shapeStyle.itemindex);
        shp.pen.color:=PenColor.ColorValue;
        shp.Brush.color:=BrushColor.ColorValue;
        shp.pen.style:=PenStyle;
        shp.brush.style:=BrushStyle;
        shp.pen.width:=strtoint(penwidth.text);
      end;
    end;
  end
  else if Sender is TAQRMemo then
  begin
    mem:=(Sender as TAQRMemo);
    if QRMemoSetupFrm=nil then
      Application.CreateForm(TQRMemoSetupFrm, QRMemoSetupFrm);
    with QRMemoSetupFrm do
    begin
      labeltext.Lines.clear;
      labeltext.Lines.addStrings(mem.Lines);
      FontSetup.Font:=mem.Font;
      labelleft.text:=inttostr(mem.left);
      labeltop.text:=inttostr(mem.top);
      labelwidth.text:=inttostr(mem.width);
      labelheight.text:=inttostr(mem.height);
      labelfield.text:=mem.fieldname;
      labelformat.text:=mem.fieldformat;
      labelfield.items.Clear;
      if Assigned(FOnGetFieldNames) then
        FOnGetFieldNames(Self,labelfield.items);
      labelfield.onExit:=Self.GetFormats;
      GetFormats(labelField);
      ShowModal;
      labelfield.onExit:=nil;
      if ModalResult=mrOk then
      begin
        mem.Font:=FontSetup.Font;
        mem.left:=strtoint(labelleft.text);
        mem.top:=strtoint(labeltop.text);
        mem.width:=strtoint(labelwidth.text);
        mem.height:=strtoint(labelheight.text);
        mem.fieldname:=labelfield.text;
        mem.fieldformat:=labelformat.text;
        mem.Lines.clear;
        mem.Lines.addStrings(labeltext.Lines);
      end;
    end;
  end
  else if Sender is TAQRBand then
  begin
    if QRBandSetupFrm=nil then
      Application.CreateForm(TQRBandSetupFrm, QRBandSetupFrm);
    with QRBandSetupFrm do
    begin
      BandHeight.text:=intToStr(((Sender as TAQRBand).Height*10+37) div 38);
      FontSetup.Font:=(Sender as TAQRBand).Font;
      ShowModal;
      if ModalResult=mrOk then
      begin
        (Sender as TAQRBand).Font:=FontSetup.Font;
        (Sender as TAQRBand).HeightAutoResize:=(strToint(BandHeight.text)*38) div 10;
      end;
    end;
  end
end;

procedure TAQREditor.ControlMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if MoveControl<>nil then
  begin
    if MoveControl is TAQRLabel then
      FrameControl.Parent:=nil;
    Screen.Cursor:=crDefault;
    MoveControl:=nil;
  end;
end;

procedure TAQREditor.ControlMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var pt:Tpoint;
    pr:TWinControl;
    i,eps:integer;
begin
  if button=mbRight then
  begin
    editMode:=0;
    PopupControl:=(Sender as TControl);
    (FindComponent('PopupDelete') as TMenuItem).Visible:=(not (Sender is TAQRBand));
    pt.x:=x;
    pt.y:=y;
    pt:=PopupControl.ClientToScreen(pt);
    PopupMenu.Popup(pt.x,pt.y);
  end
  else if (editMode=1) and (button=mbLeft) then
  begin
    if not (Sender is TAQRBand) then
    begin
      MoveControl:=(sender as Tcontrol);
      pt.x:=x;
      pt.y:=y;
      pt:=MoveControl.ClientToScreen(pt);
      MoveX:=pt.X;
      MoveY:=pt.Y;
      if MoveControl is TAQRLabel then
      begin
        FrameControl.SetBounds(MoveControl.Left,MoveControl.Top,
            MoveControl.Width,MoveControl.height);
        FrameControl.Parent:=MoveControl.Parent;
      end;
      with MoveControl do
      begin
        moveRect.Top:=top;
        moveRect.bottom:=Height+Top-1;
        moveRect.Left:=left;
        moveRect.Right:=Width+Left-1;
      end;
      eps:=3;
      if MoveControl is TAQRMemo then
        eps:=7;
      movemode:=0;
      if (abs(y - Movecontrol.height)<=eps) and
        (abs(x - Movecontrol.width)<=eps) then
          MoveMode:=3
      else if (abs(y - Movecontrol.height div 2)<=3) and
        (abs(x - Movecontrol.width)<=eps) then
          MoveMode:=1
      else if (abs(y - Movecontrol.height)<=eps) and
        (abs(x - Movecontrol.width div 2)<=3) then
          MoveMode:=2;
      if movemode=1 then
        Screen.Cursor:=crSizeWE
      else if movemode=2 then
        Screen.Cursor:=crSizeNS
      else if movemode=3 then
        Screen.Cursor:=crSizeNWSE
      else
        Screen.Cursor:=crDrag;
    end;
  end
  else if (editMode=2) and (button=mbLeft) then
  begin
    if not (Sender is TAQRBand) then
    begin
      Sender.Free;
      EditMode:=0;
    end;
  end
  else if (editMode=5) and (button=mbLeft) then
  begin
    EditControl(sender as Tcontrol);
    EditMode:=0;
  end
  else if (editMode=3) and (button=mbLeft) then
  begin
    EditMode:=0;
    pr:=TWinControl(Sender);
    if not (Sender is TAQRBand) then
      pr:=(Sender as TControl).Parent;
    pt.x:=x;
    pt.y:=y;
    pt:=(Sender as TControl).ClientToScreen(pt);
    pt:=pr.ScreenToClient(pt);
    if CreateClass=2 then
    begin
      i:=1;
      while Owner.FindComponent('QRShape'+inttostr(i))<>nil do
        i:=i+1;
      with TAQRShape.Create(Owner) do
      begin
        left:=pt.x;
        top:=pt.y;
        Name:='QRShape'+inttostr(i);
        Parent:=pr;
        OnMouseDown:=ControlMouseDown;
        OnMouseUp:=ControlMouseUp;
        OnMouseMove:=ControlMouseMove;
      end
    end
    else if CreateClass=3 then
    begin
      i:=1;
      while Owner.FindComponent('QRMemo'+inttostr(i))<>nil do
        i:=i+1;
      with TAQRMemo.Create(Owner) do
      begin
        left:=pt.x;
        top:=pt.y;
        Name:='QRMemo'+inttostr(i);
        Parent:=pr;
        OnMouseDown:=ControlMouseDown;
        OnMouseUp:=ControlMouseUp;
        OnMouseMove:=ControlMouseMove;
      end
    end
    else if CreateClass=1 then
    begin
      i:=1;
      while Owner.FindComponent('QRLabel'+inttostr(i))<>nil do
        i:=i+1;
      with TAQRLabel.Create(Owner) do
      begin
        left:=pt.x;
        top:=pt.y;
        Name:='QRLabel'+inttostr(i);
        Caption:=Name;
        Parent:=pr;
        OnMouseDown:=ControlMouseDown;
        OnMouseUp:=ControlMouseUp;
        OnMouseMove:=ControlMouseMove;
      end
    end
  end
end;

procedure TAQREditor.ControlMouseMove(Sender: TObject; Shift: TShiftState;
   X, Y: Integer);
var pt:TPoint;
    newx,newy,neww,newh,x2,y2:integer;
begin
  if MoveControl<>nil then
  begin
    pt.x:=x;
    pt.y:=y;
    pt:=(Sender as TControl).ClientToScreen(pt);
    case moveMode of
    0: begin
        newx:=MoveRect.left+pt.X-MoveX;
        newy:=MoveRect.top+pt.Y-MoveY;
        neww:=MoveRect.right-MoveRect.left+1;
        newh:=MoveRect.bottom-MoveRect.top+1;
      end;
    1: begin
        newx:=MoveRect.left;
        newy:=MoveRect.top;
        neww:=MoveRect.right-MoveRect.left+1+pt.X-MoveX;
        newh:=MoveRect.bottom-MoveRect.top+1;
      end;
    2: begin
        newx:=MoveRect.left;
        newy:=MoveRect.top;
        neww:=MoveRect.right-MoveRect.left+1;
        newh:=MoveRect.bottom-MoveRect.top+1+pt.Y-MoveY;
      end;
    3: begin
        newx:=MoveRect.left;
        newy:=MoveRect.top;
        neww:=MoveRect.right-MoveRect.left+1+pt.X-MoveX;
        newh:=MoveRect.bottom-MoveRect.top+1+pt.Y-MoveY;
      end;
    end;
    x2:=newx+neww;
    y2:=newy+newh;
    if (newx<MoveControl.Parent.Width-1) and
       (newy<MoveControl.Parent.height-1) and
       (x2>0) and (y2>0) then
    begin
      MoveControl.SetBounds(newx,newy,neww,newh);
      if MoveControl is TAQRLabel then
        FrameControl.SetBounds(newx,newy,neww,newh);
    end;
  end
end;

procedure TAQREditor.EnableEdit;
var i:integer;
begin
  EditMode:=0;
  moveControl:=nil;
  popupControl:=nil;
  with Owner do
  begin
  for i:=0 to ComponentCount-1 do
  if Components[i] is TAQRLabel then
    with Components[i] as TAQRLabel do
    begin
       OnMouseDown:=ControlMouseDown;
       OnMouseUp:=ControlMouseUp;
       OnMouseMove:=ControlMouseMove;
    end
  else if Components[i] is TAQRShape then
    with Components[i] as TAQRShape do
    begin
       OnMouseDown:=ControlMouseDown;
       OnMouseUp:=ControlMouseUp;
       OnMouseMove:=ControlMouseMove;
    end
  else if Components[i] is TAQRMemo then
    with Components[i] as TAQRMemo do
    begin
       OnMouseDown:=ControlMouseDown;
       OnMouseUp:=ControlMouseUp;
       OnMouseMove:=ControlMouseMove;
    end
  else if Components[i] is TAQRBand then
    with Components[i] as TAQRBand do
    begin
       OnMouseDown:=ControlMouseDown;
       OnMouseUp:=ControlMouseUp;
       OnMouseMove:=ControlMouseMove;
    end;
  end;
end;

procedure TAQREditor.DisableEdit;
var i:integer;
begin
  EditMode:=0;
  moveControl:=nil;
  popupControl:=nil;
  if Owner=nil then Exit;
  with Owner do
  begin
  for i:=0 to ComponentCount-1 do
  if Components[i] is TAQRLabel then
    with Components[i] as TAQRLabel do
    begin
       OnMouseDown:=nil;
       OnMouseUp:=nil;
       OnMouseMove:=nil;
    end
  else if Components[i] is TAQRShape then
    with Components[i] as TAQRShape do
    begin
       OnMouseDown:=nil;
       OnMouseUp:=nil;
       OnMouseMove:=nil;
    end
  else if Components[i] is TAQRMemo then
    with Components[i] as TAQRMemo do
    begin
       OnMouseDown:=nil;
       OnMouseUp:=nil;
       OnMouseMove:=nil;
    end
  else if Components[i] is TAQRBand then
    with Components[i] as TAQRBand do
    begin
       OnMouseDown:=nil;
       OnMouseUp:=nil;
       OnMouseMove:=nil;
    end;
  end;
end;

procedure TAQREditor.FileCloseClick(Sender: TObject);
begin
  (Owner as TForm).Close;
  DisableEdit;
  (Owner as TForm).Menu:=nil;
end;

procedure TAQREditor.EditDeleteClick(Sender: TObject);
begin
  EditMode:=2;
end;

procedure TAQREditor.EditMoveClick(Sender: TObject);
begin
  EditMode:=1;
end;

procedure TAQREditor.FileSaveClick(Sender: TObject);
Var Form:TForm;
    oldVis:Boolean;
begin
  Form:=Owner as TForm;
  DisableEdit;
  with Form do
  begin
    Menu:=nil;
    RemoveComponent(Self);
    oldVis:=Visible;
    Visible:=False;
  end;
  WriteComponentResFile(FFileName,Form);
  with Form do
  begin
    InsertComponent(Self);
    Menu:=MainMenu;
    Visible:=oldVis;
  end;
  EnableEdit;
end;

procedure TAQREditor.LoadReport;
Var Form:TForm;
    oldVis:Boolean;
begin
  if FileExists(FFileName) then
  begin
    Form:=(Owner as TForm);
    with Form do
    begin
      OldVis:=Visible;
      RemoveComponent(Self);
      while ComponentCount>0 do
        Components[0].Free;
    end;
    ReadComponentResFile(FFileName,Form);
    Form.InsertComponent(Self);
    Form.Visible:=OldVis;
  end;
end;

procedure TAQREditor.FileOpenClick(Sender: TObject);
Var Form:TForm;
begin
  Form:=Owner as TForm;
  LoadReport;
  EnableEdit;
  Form.menu:=mainmenu;
end;

procedure TAQREditor.PopupMoveClick(Sender: TObject);
begin
  EditMode:=1;
end;

procedure TAQREditor.EditPropertyClick(Sender: TObject);
begin
  EditMode:=5;
end;

procedure TAQREditor.PopupChangeClick(Sender: TObject);
begin
  EditControl(PopupControl);
end;

procedure TAQREditor.PopupDeleteClick(Sender: TObject);
begin
  if not (PopupControl is TAQRBand) then
  begin
    PopupControl.Free;
    popupControl:=nil
  end;
end;

procedure TAQREditor.FilePrintClick(Sender: TObject);
var QuickReport:{$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF};
    i:integer;
begin
  if assigned(FOnPrintPreview) then
    FOnPrintPreview(Self);
  QuickReport:=nil;
  with Owner do
    for i:=0 to ComponentCount-1 do
      if Components[i] is {$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF} then
      begin
        QuickReport:=Components[i] as {$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF};
        break;
      end;
  if QuickReport<>nil then
    QuickReport.Preview;
end;

procedure TAQREditor.EditCreateLabelClick(Sender: TObject);
begin
  EditMode:=3;
  CreateClass:=1;
end;

procedure TAQREditor.EditCreateShapeClick(Sender: TObject);
begin
  EditMode:=3;
  CreateClass:=2;
end;

procedure TAQREditor.PopupToBackClick(Sender: TObject);
begin
   PopupControl.SendtoBack;
end;

procedure TAQREditor.PopupToFrontClick(Sender: TObject);
begin
  PopupControl.BringtoFront;
end;

procedure TAQREditor.FilePrinterClick(Sender: TObject);
var PrinterSetupDialog1:TPrinterSetupDialog;
begin
   PrinterSetupDialog1:=TPrinterSetupDialog.Create(Owner);
   PrinterSetupDialog1.Execute;
   PrinterSetupDialog1.Free;
end;

procedure TAQREditor.FilePageClick(Sender: TObject);
var FirstQRBand:TQRBand;
    QuickReport:{$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF};
    i:integer;
begin
  FirstQRBand:=nil;
  QuickReport:=nil;
  with Owner do
  begin
    for i:=0 to ComponentCount-1 do
      if Components[i] is TQRBand then
      begin
        if (Components[i] as TQRBand).BandType=rbPageHeader then
          FirstQRBand:=(Components[i] as TQRBand);
      end
      else if Components[i] is {$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF} then
        QuickReport:=Components[i] as {$IFDEF QR20}TQuickRep{$ELSE}TQuickReport{$ENDIF};
  end;
  if PageSetupFrm=nil then
      Application.CreateForm(TPageSetupFrm, PageSetupFrm);
  with PageSetupFrm do
  begin
    {$IFDEF QR20}
    PageLeft.text:=IntToStr(trunc(QuickReport.Page.LeftMargin));
    PageTop.text:=IntToStr(trunc(QuickReport.Page.TopMargin));
    {$ELSE}
    PageLeft.text:=intToStr(QuickReport.LeftMarginMM);
    if FirstQRBand=nil then
      PageTop.text:='0'
    else
      PageTop.text:=intToStr((FirstQRBand.Height*10+37) div 38);
    {$ENDIF}
    ShowModal;
  end;
  if PageSetupFrm.ModalResult=mrOk then
  begin
    {$IFDEF QR20}
    QuickReport.Page.LeftMargin:=strToInt(PageSetupFrm.PageLeft.text);
    QuickReport.Page.TopMargin:=strToInt(PageSetupFrm.PageTop.text);
    {$ELSE}
    QuickReport.LeftMarginMM:=strToint(PageSetupFrm.PageLeft.text);
    if FirstQRBand<>nil then
      FirstQRBand.Height:=(strToint(PageSetupFrm.PageTop.text)*38) div 10;
    {$ENDIF}
  end;
end;

procedure TAQREditor.EditCreateMemoClick(Sender: TObject);
begin
  EditMode:=3;
  CreateClass:=3;
end;

procedure TAQREditor.EditReport;
var PopupItems:array [0..4] of TMenuItem;
    FileItems:array [0..6] of TMenuItem;
    EditItems:array [0..3] of TMenuItem;
    MainItems:array [0..1] of TMenuItem;
    CreateItems:array [0..2] of TMenuItem;
begin
  if PopupMenu=nil then
  begin
    PopupItems[0]:=NewItem(LoadStr(SPosSize),0,False,True,PopupMoveClick,
       0,'PopupMove');
    PopupItems[1]:=NewItem(LoadStr(SProp),0,False,True,PopupChangeClick,
       0,'PopupChange');
    PopupItems[2]:=NewItem(LoadStr(SDelete),0,False,True,PopupDeleteClick,
       0,'PopupDelete');
    PopupItems[3]:=NewItem(LoadStr(SToBack),0,False,True,PopupToBackClick,
       0,'PopupToBack');
    PopupItems[4]:=NewItem(LoadStr(SToFront),0,False,True,PopupToFrontClick,
       0,'PopupToFront');
    PopupMenu:=NewPopupMenu(Self,'AQREditorPopupMenu',paLeft,True,PopupItems);
  end;
  if MainMenu=nil then
  begin
    FileItems[0]:=NewItem(LoadStr(SOpen),0,False,True,FileOpenClick,
       0,'FileOpen');
    FileItems[1]:=NewItem(LoadStr(SSave),0,False,True,FileSaveClick,
       0,'FileSave');
    FileItems[2]:=NewItem(LoadStr(SPrinter),0,False,True,FilePrinterClick,
       0,'FilePrinter');
    FileItems[3]:=NewItem(LoadStr(SPrint),0,False,True,FilePrintClick,
       0,'FilePrint');
    FileItems[4]:=NewItem(LoadStr(SPageProp),0,False,True,FilePageClick,
       0,'FilePage');
    FileItems[5]:=NewLine;
    FileItems[6]:=NewItem(LoadStr(SClose),0,False,True,FileCloseClick,
       0,'FileClose');
    CreateItems[0]:=NewItem(LoadStr(SCreateLabel),0,False,True,EditCreateLabelClick,
       0,'EditCreateLabel');
    CreateItems[1]:=NewItem(LoadStr(SCreateShape),0,False,True,EditCreateShapeClick,
       0,'EditCreateShape');
    CreateItems[2]:=NewItem(LoadStr(SCreateMemo),0,False,True,EditCreateMemoClick,
       0,'EditCreateMemo');
    EditItems[0]:=NewItem(LoadStr(SPosSize),0,False,True,EditMoveClick,
       0,'EditMove');
    EditItems[1]:=NewItem(LoadStr(SProp),0,False,True,EditPropertyClick,
       0,'EditProperty');
    EditItems[2]:=NewSubMenu(LoadStr(SCreate),0,'EditCreate',CreateItems);
    EditItems[3]:=NewItem(LoadStr(SDelete),0,False,True,EditDeleteClick,
       0,'EditDelete');
    MainItems[0]:=NewSubMenu(LoadStr(SFile),0,'FileMenu',FileItems);
    MainItems[1]:=NewSubMenu(LoadStr(SEdit),0,'EditMenu',EditItems);
    MainMenu:=NewMenu(Self,'AQREditorMainMenu',MainItems);
  end;
  with Owner as TForm do
  begin
    menu:=mainmenu;
    enableEdit;
    if Assigned(FOnCreateMenu) then
      FOnCreateMenu(Self);
    Show;
  end
end;

procedure TAQREditor.DefaultFormatValue(const FormatStr:string;
      ValueType:TFormatValueType; ValuePtr:Pointer; var ResultStr:string);
type PString=^String;
     PInteger=^Longint;
     PFloat=^Extended;
     PDateTime=^TDateTime;
begin
  case ValueType of
  fvInteger: ResultStr:=FormatFloat(FormatStr,(PInteger(ValuePtr)^));
  fvFloat: ResultStr:=FormatFloat(FormatStr,(PFloat(ValuePtr)^));
  fvDateTime: ResultStr:=FormatDateTime(FormatStr,(PDateTime(ValuePtr)^));
  fvString: ResultStr:=(PString(ValuePtr)^);
  end;
end;

procedure TAQREditor.DoFormatValue(const FormatStr:string;
      ValueType:TFormatValueType; ValuePtr:Pointer; var ResultStr:string);
begin
  if Assigned(FOnFormatValue) then
    FOnFormatValue(Self,FormatStr,ValueType,ValuePtr,ResultStr)
  else
    DefaultFormatValue(FormatStr,ValueType,ValuePtr,ResultStr);
end;

procedure TAQREditor.SetFieldInteger(const FieldName:string; Value:longint);
begin
  SetFieldValue(FieldName,fvInteger,Pointer(@Value));
end;

procedure TAQREditor.SetFieldFloat(const FieldName:string; Value:extended);
begin
  SetFieldValue(FieldName,fvFloat,Pointer(@Value));
end;

procedure TAQREditor.SetFieldString(const FieldName:string; const Value:string);
begin
  SetFieldValue(FieldName,fvString,Pointer(@Value));
end;

procedure TAQREditor.SetFieldDateTime(const FieldName:string; Value:TDateTime);
begin
  SetFieldValue(FieldName,fvDateTime,Pointer(@Value));
end;

procedure TAQREditor.SetFieldValue(const FieldName:string; ValueType:TFormatValueType; ValuePtr:Pointer);
var i:integer;
    Res:string;
begin
  with owner do
    for i:=0 to ComponentCount-1 do
    begin
      if (Components[i] is TAQRLabel) then
      begin
        if CompareText((Components[i] as TAQRLabel).FieldName,FieldName)=0 then
        begin
          DoFormatValue((Components[i] as TAQRLabel).FieldFormat,ValueType,ValuePtr,Res);
          (Components[i] as TAQRLabel).Caption:=Res;
        end;
      end
      else if (Components[i] is TAQRMemo) then
      begin
        if CompareText((Components[i] as TAQRMemo).FieldName,FieldName)=0 then
        begin
          DoFormatValue((Components[i] as TAQRMemo).FieldFormat,ValueType,ValuePtr,Res);
          with (Components[i] as TAQRMemo).Lines do
          begin
            Clear;
            Add(Res);
          end;
        end;
      end;
    end;
end;

initialization
  Classes.RegisterClass(TAQRLabel);
  Classes.RegisterClass(TAQRMemo);
  Classes.RegisterClass(TAQRShape);
  Classes.RegisterClass(TAQRBand);
end.
